/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.camel.component.sqlplus;

import metridoc.utils.Assert;
import org.apache.camel.Endpoint;
import org.apache.camel.impl.DefaultComponent;
import org.apache.camel.impl.ProcessorEndpoint;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.sql.DataSource;
import java.util.Map;

public class SqlPlusComponent extends DefaultComponent {
	
	private static final String FETCH_SIZE_MIN = "min";
	private static final Logger LOG = LoggerFactory.getLogger(SqlPlusComponent.class);
    private static final int DEFAULT_FETCH_SIZE = 10;
    private static final int DEFAULT_BATCH_SIZE = 10;


    @Override
	protected Endpoint createEndpoint(String uri, String remaining, Map<String, Object> parameters) throws Exception {
		String dataSourceKey = getAndRemoveParameter(parameters, "dataSource", String.class);
		Assert.notNull(dataSourceKey, "the variable 'dataSource' must be set on the sqlplus endpoint");
        DataSource dataSource = getCamelContext().getRegistry().lookup(dataSourceKey, DataSource.class);
		remaining = remaining.trim();
		if (remaining.endsWith(".sql")) {
			return createFileEndpoint(uri, remaining, parameters, dataSource);
		} else {
			return createSqlPlusEndpoint(uri, remaining, parameters, dataSource);
		} 
	}

	private Endpoint createSqlPlusEndpoint(String uri, String query, Map<String, Object> parameters,
			DataSource dataSource) throws Exception {
		int fetchSize = getFetchSize(parameters);
		int batchSize = DEFAULT_BATCH_SIZE;
		if (parameters.containsKey("batchSize")) {
            String batchSizeString = (String) parameters.remove("batchSize");
            try {
                batchSize = Integer.parseInt(batchSizeString);
            } catch (NumberFormatException ex) {
                String message = "batchSize is " + batchSizeString + ", which is not an integer, please set greater " +
                        "than 0 or don't include and use the defaults";
                throw new IllegalArgumentException(message);
            }
            Assert.isTrue(batchSize > 0, "batchSize was set at " + batchSize + ", please make it greater than 0");
		}
		boolean detailedOutput = Boolean.parseBoolean((String) parameters.remove("detailedOutput"));
		
		SqlPlusEndpoint endpoint = new SqlPlusEndpoint(uri, this, dataSource, query);
		endpoint.setFetchSize(fetchSize);
		endpoint.setBatchSize(batchSize);
		endpoint.setDetailedOutput(detailedOutput);
		return endpoint;
	}

	private Endpoint createFileEndpoint(String uri, String filePath, Map<String, Object> parameters,
			DataSource dataSource) throws Exception {
		return new ProcessorEndpoint(uri, getCamelContext(), new SqlFileRouteProcessor(filePath, dataSource));
	}

	private int getFetchSize(Map<String, Object> parameters) {
		int fetchSize = DEFAULT_FETCH_SIZE;
		String fetchSizeString = getAndRemoveParameter(parameters, "fetchSize", String.class);

        if (FETCH_SIZE_MIN.equals(fetchSizeString)) {
			fetchSize = Integer.MIN_VALUE;
		} else if (fetchSizeString != null){
			try {
				fetchSize = Integer.parseInt(fetchSizeString);
			} catch (NumberFormatException e) {
				String message = "fetchSize is " + fetchSizeString + ", which is not an integer, please set greater " +
                        "than 0, 'min' to enable mysql streaming or don't include and use the defaults";
                throw new IllegalArgumentException(message);
			}
		}
		LOG.debug("Fetch size is " + fetchSize);
		return fetchSize;
	}
}
